@@ -18,7 +18,7 @@ class DistilBertSelfAttentionAdaptersModule(BertSelfOutputAdaptersMixin, nn.Modu
         self.config = parent.config
 
     @property
-    def layer_norm(self):
+    def transformer_layer_norm(self):
         return self.parent.sa_layer_norm
 
 
@@ -32,7 +32,7 @@ class DistilBertOutputAdaptersModule(BertOutputAdaptersMixin, nn.Module):
         self.config = parent.config
 
     @property
-    def layer_norm(self):
+    def transformer_layer_norm(self):
         return self.parent.output_layer_norm
 
 
@@ -44,6 +44,7 @@ class DistilBertTransfomerBlockAdaptersMixin:
         self.output_adapters = DistilBertOutputAdaptersModule(self)
         self.attention_adapters._init_adapter_modules()
         self.output_adapters._init_adapter_modules()
+        self.register_forward_pre_hook(self._adapter_block_pre_hook)
 
     def add_fusion_layer(self, adapter_names):
         self.attention_adapters.add_fusion_layer(adapter_names)
@@ -65,6 +66,13 @@ class DistilBertTransfomerBlockAdaptersMixin:
         self.attention_adapters.enable_adapters(adapter_names, unfreeze_adapters, unfreeze_attention)
         self.output_adapters.enable_adapters(adapter_names, unfreeze_adapters, unfreeze_attention)
 
+    # Makes sure the "parent" reference always points to the correct module.
+    # This is especially relevant when using torch data parallelism.
+    @staticmethod
+    def _adapter_block_pre_hook(module, input_tensors):
+        object.__setattr__(module.attention_adapters, "parent", module)
+        object.__setattr__(module.output_adapters, "parent", module)
+
 
 class DistilBertTransformerAdaptersMixin(BertEncoderAdaptersMixin):
     """Adds adapters to the Transformer module of DistilBert."""
